-----------------------------------------------------------------------------------------------
-- Client Lua Script for Heckler - by Ciuine - GPLv3
-----------------------------------------------------------------------------------------------
 
require "Window"
require "ICCommLib"
require "Apollo"
require "GameLib"
require "GroupLib"
require "AbilityBook"
 
-----------------------------------------------------------------------------------------------
-- Heckler Module Definition
-----------------------------------------------------------------------------------------------
local Heckler = {} 
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function Heckler:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self 

    return o
end

function Heckler:Init()
	local bHasConfigureFunction = true
	local strConfigureButtonText = "Heckler"
	local tDependencies = {
	}
    Apollo.RegisterAddon(self, bHasConfigureFunction, strConfigureButtonText, tDependencies)
end
 
-----------------------------------------------------------------------------------------------
-- Heckler OnLoad
-----------------------------------------------------------------------------------------------
function Heckler:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("Heckler.xml")
end

-----------------------------------------------------------------------------------------------
-- Heckler OnDocLoaded
-----------------------------------------------------------------------------------------------
function Heckler:GetAsyncLoadStatus()

	if self.xmlDoc ~= nil and self.xmlDoc:IsLoaded() then
		self.wndMain = Apollo.LoadForm(self.xmlDoc, "HecklerForm", nil, self)
		self.wndOptions = Apollo.LoadForm(self.xmlDoc, "OptionsForm", nil, self)
		self.wndMain:SetOpacity(.8)
		self.tUnitList = {}
		self.tUnitCont = {}
		self.tSortedList = {}
		self.bIsInGroup = GroupLib.GetMemberCount() > 0
		if self.bIsInGroup then Heckler:NotifyGroup() end
		self.strCurrentActive = ""
		self.msgDelay = 0
		self.wndMain:Show(self.bIsInGroup)
		self.wndOptions:Show(false)
		if self.tAnchors ~= nil then
		self.wndMain:SetAnchorOffsets(unpack(self.tAnchors))
		end
		self.wndOptions:FindChild("Width"):FindChild("EditBox"):SetText(self.nWidth)
		self.wndOptions:FindChild("Height"):FindChild("EditBox"):SetText(self.nHeight)
		self.wndOptions:FindChild("IconWidth"):FindChild("EditBox"):SetText(self.nIconWidth)
		self.wndOptions:FindChild("UpdateFreq"):FindChild("EditBox"):SetText(self.nUpdateFreq)

		self.nNext = 1
		self.timer = ApolloTimer.Create(self.nUpdateFreq, true, "OnTimer", self)
		Apollo.RegisterTimerHandler("OneSecTimer", "SubtractCD", self)
		Apollo.RegisterEventHandler("Group_Join", "NotifyGroup", self)
		Apollo.RegisterEventHandler("Group_Add", "NotifyGroup", self)
		Apollo.RegisterEventHandler("Group_Remove", "RemoveMember", self)
		Apollo.RegisterEventHandler("Group_Updated", "NotifyGroup", self)
		Apollo.RegisterEventHandler("Group_Left", "LeftGroup", self)
		Apollo.RegisterEventHandler("CombatLogCCState", "CCState", self)
		Apollo.RegisterEventHandler("CombatLogModifyInterruptArmor", "CCState", self)
		Apollo.RegisterSlashCommand("heckle", "OnSlash", self)
		Apollo.RegisterSlashCommand("heckleopt", "OnConfigure", self)
		Apollo.RegisterEventHandler("InterfaceMenuListHasLoaded", "OnInterfaceMenuListHasLoaded", self)
		Apollo.RegisterEventHandler("HecklerConfigure", "OnConfigure", self)
		return Apollo.AddonLoadStatus.Loaded
	end
	return Apollo.AddonLoadStatus.Loading
end

function Heckler:OnSave(eType)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
	local tSavedData = {}
	tSavedData.tAnchorOffsets = {self.wndMain:GetAnchorOffsets()}
	tSavedData.nWidth = self.nWidth
	tSavedData.nHeight = self.nHeight
	tSavedData.nIconWidth = self.nIconWidth
	tSavedData.nUpdateFreq = self.nUpdateFreq
	return tSavedData
end

function Heckler:OnRestore(eType, tSavedData)
	if eType ~= GameLib.CodeEnumAddonSaveLevel.Account then
		return
	end
		
	if tSavedData.tAnchorOffsets then	
		self.tAnchors = tSavedData.tAnchorOffsets
	end
	self.nWidth = tSavedData.nWidth and tSavedData.nWidth or 134
	self.nHeight = tSavedData.nHeight and tSavedData.nHeight or 15
	self.nIconWidth = tSavedData.nIconWidth and tSavedData.nIconWidth or 22
	self.nUpdateFreq = tSavedData.nUpdateFreq and tSavedData.nUpdateFreq or 1
end

function Heckler:OnSlash()
	if Heckler.tUnitCont then
		for i, v in pairs(Heckler.tUnitCont) do
			v:Destroy()
		end
		Heckler.tUnitList = {}
	end
	if Heckler.shareChannel and self.bIsInGroup then
		local msg = {
		["Sender"] = GameLib.GetPlayerUnit():GetName(),
		["Text"] = "Heckle",
		}
		Heckler.shareChannel:SendMessage(msg)
		Heckler:ShareHecklerInfoReceipt("", msg)
	end
end

function Heckler:OnInterfaceMenuListHasLoaded()
	Event_FireGenericEvent("InterfaceMenuList_NewAddOn", "Heckler", {"HecklerConfigure", "", ""})
end
-----------------------------------------------------------------------------------------------
-- Heckler Functions
-----------------------------------------------------------------------------------------------
function Heckler:CreateNewUnit(self, unitName, isPlayer)
	self.tUnitList[unitName] = {}
	self.tUnitCont[unitName] = Apollo.LoadForm(self.xmlDoc, "InnerCont", self.wndMain, self)
	self.tUnitCont[unitName]:FindChild("Text"):SetText(" " .. unitName)
	self.tUnitCont[unitName]:SetData(isPlayer and "`" or unitName)
	self.tUnitCont[unitName]:Show(true)
	local nCount = GroupLib.GetMemberCount()
	local l, t, r, b = self.wndMain:GetAnchorOffsets()
	self.wndMain:SetAnchorOffsets(l, t, l+self.nWidth, t+(self.nHeight*nCount))
	l, t, r, b = self.tUnitCont[unitName]:GetAnchorOffsets()
	self.tUnitCont[unitName]:SetAnchorOffsets(l, t, r, t+self.nHeight)
	self.wndMain:ArrangeChildrenVert(0, function(a, b) if a and b then return a:GetData() < b:GetData() end end)
end

function Heckler:OnTimer()
	if Heckler.shareChannel and self.bIsInGroup then
		local unitPlayer = GameLib.GetPlayerUnit()
		if unitPlayer then
			local unitName = unitPlayer:GetName()
			local tLASActive = {}
			local tLAS = ActionSetLib.GetCurrentActionSet()
			for i = 1, 8 do
				tLASActive[i] = tLAS[i]
			end
			for i, v in pairs(AbilityBook.GetAbilitiesList()) do
				if v.nId == 30160 or v.nId == 19355 then
					local nCDR = v.tTiers[v.nCurrentTier].splObject:GetCooldownRemaining()
					if nCDR > v.tTiers[v.nCurrentTier].splObject:GetCooldownTime() - (self.nUpdateFreq*1.2) or self.bHeckleRequest then
						if self.tUnitList[unitName] == nil then
							self:CreateNewUnit(self, unitName, true)
						end
						if self.tUnitList[unitName][v.strName] == nil then
							self.tUnitList[unitName][v.strName] = {nCD = math.floor(nCDR - (self.nUpdateFreq*2)); nTier = v.nCurrentTier}
							local newInner = Apollo.LoadForm(self.xmlDoc, "InnerForm", self.tUnitCont[unitName]:FindChild("Icons"), self)
							newInner:Show(true)
							local l, t, r, b = newInner:GetAnchorOffsets()
							newInner:SetAnchorOffsets(l, t, self.nIconWidth, b)
							newInner:FindChild("Icon"):SetSprite(v.tTiers[v.nCurrentTier].splObject:GetIcon())
							self.tUnitList[unitName][v.strName].widg = newInner
							self.tUnitCont[unitName]:FindChild("Icons"):ArrangeChildrenHorz(2)
						else
							self.tUnitList[unitName][v.strName]["nCD"] = math.floor(nCDR - (self.nUpdateFreq*2))
							self.tUnitList[unitName][v.strName]["nTier"] = v.nCurrentTier
						end
						local msg = {
						["Sender"] = unitName,
						["Text"] = "UI",
						["Name"] = v.strName,
						["Time"] = self.tUnitList[unitName][v.strName]["nCD"],
						["Tier"] = v.nCurrentTier,
						["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
						Heckler.shareChannel:SendMessage(msg)
						Heckler:ShareHecklerInfoReceipt("", msg)
					end
					if self.tUnitList[unitName] and self.tUnitList[unitName][v.strName] and nCDR > 0 and nCDR < self.tUnitList[unitName][v.strName]["nCD"] - (self.nUpdateFreq*.5) then
						self.tUnitList[unitName][v.strName]["nCD"] = math.floor(nCDR - (self.nUpdateFreq*2))
						local msg = {
						["Sender"] = unitName,
						["Text"] = "UI",
						["Name"] = v.strName,
						["Time"] = math.floor(nCDR),
						["Tier"] = v.nCurrentTier,
						["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
						Heckler.shareChannel:SendMessage(msg)
						Heckler:ShareHecklerInfoReceipt("", msg)
					end
					if self.tUnitList[unitName] and self.tUnitList[unitName][v.strName] and nCDR == 0 and self.tUnitList[unitName][v.strName]["nCD"] > 0 then
						self.tUnitList[unitName][v.strName]["nCD"] = 0
						local msg = {
						["Sender"] = unitName,
						["Text"] = "UI",
						["Name"] = v.strName,
						["Time"] = math.floor(nCDR),
						["Tier"] = v.nCurrentTier,
						["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
						Heckler.shareChannel:SendMessage(msg)
						Heckler:ShareHecklerInfoReceipt("", msg)
					end
				end

				for j, x in pairs(tLASActive) do
					if (v.nId == x) and string.find(string.lower(v.strAbilityDescription), "interrupt") and string.find(string.lower(v.strAbilityDescription), "destroy") then
						local nCDR = v.tTiers[v.nCurrentTier].splObject:GetCooldownRemaining()
						if nCDR > v.tTiers[v.nCurrentTier].splObject:GetCooldownTime() - (self.nUpdateFreq*1.2) or self.bHeckleRequest then
							if self.tUnitList[unitName] == nil then
								self:CreateNewUnit(self, unitName, true)
							end
							if self.tUnitList[unitName][v.strName] == nil then
								self.tUnitList[unitName][v.strName] = {nCD = math.floor(nCDR - (self.nUpdateFreq*2)); nTier = v.nCurrentTier}
								local newInner = Apollo.LoadForm(self.xmlDoc, "InnerForm", self.tUnitCont[unitName]:FindChild("Icons"), self)
								newInner:Show(true)
								local l, t, r, b = newInner:GetAnchorOffsets()
								newInner:SetAnchorOffsets(l, t, self.nIconWidth, b)
								newInner:FindChild("Icon"):SetSprite(v.tTiers[v.nCurrentTier].splObject:GetIcon())
								self.tUnitList[unitName][v.strName].widg = newInner
								self.tUnitCont[unitName]:FindChild("Icons"):ArrangeChildrenHorz(2)
							else
								self.tUnitList[unitName][v.strName]["nCD"] = math.floor(nCDR - (self.nUpdateFreq*2))
								self.tUnitList[unitName][v.strName]["nTier"] = v.nCurrentTier
							end
							local msg = {
							["Sender"] = unitName,
							["Text"] = "UI",
							["Name"] = v.strName,
							["Time"] = self.tUnitList[unitName][v.strName]["nCD"],
							["Tier"] = v.nCurrentTier,
							["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
							Heckler.shareChannel:SendMessage(msg)
							Heckler:ShareHecklerInfoReceipt("", msg)
						end
						if self.tUnitList[unitName] and self.tUnitList[unitName][v.strName] and nCDR > 0 and nCDR < self.tUnitList[unitName][v.strName]["nCD"] - (self.nUpdateFreq*.5) then
							self.tUnitList[unitName][v.strName]["nCD"] = math.floor(nCDR - (self.nUpdateFreq*2))
							local msg = {
							["Sender"] = unitName,
							["Text"] = "UI",
							["Name"] = v.strName,
							["Time"] = math.floor(nCDR),
							["Tier"] = v.nCurrentTier,
							["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
							Heckler.shareChannel:SendMessage(msg)
							Heckler:ShareHecklerInfoReceipt("", msg)
						end
						if self.tUnitList[unitName] and self.tUnitList[unitName][v.strName] and nCDR == 0 and self.tUnitList[unitName][v.strName]["nCD"] > 0 then
							self.tUnitList[unitName][v.strName]["nCD"] = 0
							local msg = {
							["Sender"] = unitName,
							["Text"] = "UI",
							["Name"] = v.strName,
							["Time"] = math.floor(nCDR),
							["Tier"] = v.nCurrentTier,
							["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
							Heckler.shareChannel:SendMessage(msg)
							Heckler:ShareHecklerInfoReceipt("", msg)
						end
						if self.bHeckleRequest then
							local msg = {
							["Sender"] = unitName,
							["Text"] = "UI",
							["Name"] = v.strName,
							["Time"] = math.floor(nCDR),
							["Tier"] = v.nCurrentTier,
							["Object"] = v.tTiers[v.nCurrentTier].splObject:GetIcon()}
							Heckler.shareChannel:SendMessage(msg)
							Heckler:ShareHecklerInfoReceipt("", msg)
						end
					end
				end
			end
			if self.bHeckleRequest then
				self.bHeckleRequest = false
			end
		end
	end
end

function Heckler:CCState(p)
	if p.unitTarget and p.unitCaster and ((p.nInterruptArmorHit and p.nInterruptArmorHit > 0) or p.splCallingSpell:GetName() == "Arcane Shock") and p.unitTarget:IsTaggedByMe() and GroupLib.GetMemberCount() > 0 then
		local bIsInGroup = false
		local unitCaster = p.unitCaster:GetName()
		for i = 1, GroupLib.GetMemberCount() do
			if GroupLib.GetGroupMember(i).strCharacterName == unitCaster then
				bIsInGroup = true
			end
		end
		if bIsInGroup and self.tUnitList and self.tUnitList[unitCaster] == nil then
			self:CreateNewUnit(self, unitCaster, false)
		end
		if bIsInGroup and self.tUnitList and self.tUnitList[unitCaster] then
			local strSpell = p.splCallingSpell:GetName()
			if self.tUnitList[unitCaster] and self.tUnitList[unitCaster][strSpell] == nil then
				self.tUnitList[unitCaster][strSpell] = {}
				if p.splCallingSpell:GetCooldownTime() then
					self.tUnitList[unitCaster][strSpell].nCD = math.floor(p.splCallingSpell:GetCooldownTime()-(self.nUpdateFreq))
				else
					self.tUnitList[unitCaster][strSpell].nCD = 20
				end
				local newInner = Apollo.LoadForm(self.xmlDoc, "InnerForm", self.tUnitCont[unitCaster]:FindChild("Icons"), self)
				newInner:Show(true)
				local l, t, r, b = newInner:GetAnchorOffsets()
				newInner:SetAnchorOffsets(l, t, self.nIconWidth, b)
				if p.splCallingSpell:GetIcon() then
					newInner:FindChild("Icon"):SetSprite(p.splCallingSpell:GetIcon())
				end
				self.tUnitList[unitCaster][strSpell].widg = newInner
				self.tUnitCont[unitCaster]:FindChild("Icons"):ArrangeChildrenHorz(2)
				self.tUnitCont[unitCaster]:Show(true)
				self.tUnitList[unitCaster][strSpell].widg:FindChild("Icon"):FindChild("Text"):Show(true)
			elseif self.tUnitList and self.tUnitList[unitCaster] and self.tUnitList[unitCaster][strSpell] then
				self.tUnitCont[unitCaster]:FindChild("Icons"):ArrangeChildrenHorz(2)
				if self.tUnitList[unitCaster][strSpell].nCD < 10 then
					if p.splCallingSpell:GetCooldownTime() then
						self.tUnitList[unitCaster][strSpell].nCD = math.floor(p.splCallingSpell:GetCooldownTime()-(self.nUpdateFreq))
					else
						self.tUnitList[unitCaster][strSpell].nCD = 20
					end
				end
				self.tUnitList[unitCaster][strSpell].widg:FindChild("Icon"):FindChild("Text"):Show(true)
			end
			self.wndMain:Show(true)
		end
	end
end

function Heckler:SubtractCD()
	if Heckler.shareChannel and self.bIsInGroup then
		self.wndMain:Show(true)
		for i, v in pairs(self.tUnitList) do
			if v ~= nil then
				for j, x in pairs(v) do
					if x.nCD > 0 then
						x.widg:FindChild("Icon"):FindChild("Text"):SetText(self.tUnitList[i][j].nCD)
						self.tUnitList[i][j].nCD = x.nCD - 1
						x.widg:FindChild("Icon"):FindChild("Text"):Show(true)
					elseif x.nCD == 0 or x.nCD == -1 then
						self.tUnitList[i][j].nCD = x.nCD - 1
						x.widg:FindChild("Icon"):FindChild("Text"):Show(false)
					end
				end
			end
		end
	end
end

function Heckler:ShareHecklerInfoReceipt(chan, msg)
	if GroupLib.GetMemberCount() > 0 then
		local bIsInGroup = false
		for i = 1, GroupLib.GetMemberCount() do
			if GroupLib.GetGroupMember(i).strCharacterName == msg.Sender then
				bIsInGroup = true
			end
		end
		if bIsInGroup then
			if msg.Sender ~= nil then
				if msg.Text == "UI" then
					if self.tUnitList and self.tUnitList[msg.Sender] == nil then
						self:CreateNewUnit(self, msg.Sender, false)
					end
					if self.tUnitList and self.tUnitList[msg.Sender] then
						if self.tUnitList[msg.Sender] and self.tUnitList[msg.Sender][msg.Name] == nil then
							self.tUnitList[msg.Sender][msg.Name] = {}
							self.tUnitList[msg.Sender][msg.Name].nCD = math.floor(msg.Time - (self.nUpdateFreq))
							local newInner = Apollo.LoadForm(self.xmlDoc, "InnerForm", self.tUnitCont[msg.Sender]:FindChild("Icons"), self)
							newInner:Show(true)
							local l, t, r, b = newInner:GetAnchorOffsets()
							newInner:SetAnchorOffsets(l, t, self.nIconWidth, b)
							newInner:FindChild("Icon"):SetSprite(msg.Object)
							self.tUnitList[msg.Sender][msg.Name].widg = newInner
							self.tUnitCont[msg.Sender]:FindChild("Icons"):ArrangeChildrenHorz(2)
							self.tUnitCont[msg.Sender]:Show(true)
							self.tUnitList[msg.Sender][msg.Name].widg:FindChild("Icon"):FindChild("Text"):SetText(msg.Time)
							self.tUnitList[msg.Sender][msg.Name].widg:FindChild("Icon"):FindChild("Text"):Show(true)
						elseif self.tUnitList and self.tUnitList[msg.Sender] and self.tUnitList[msg.Sender][msg.Name] then
							self.tUnitCont[msg.Sender]:FindChild("Icons"):ArrangeChildrenHorz(2)
							self.tUnitList[msg.Sender][msg.Name].nCD = math.floor(msg.Time - (self.nUpdateFreq))
							self.tUnitList[msg.Sender][msg.Name].widg:FindChild("Icon"):FindChild("Text"):SetText(msg.Time)
							self.tUnitList[msg.Sender][msg.Name].widg:FindChild("Icon"):FindChild("Text"):Show(true)
						end
					end
				elseif msg.Text == "Heckle" then
					self.bHeckleRequest = true
				end
			end
		end
	end
end

function Heckler:NotifyGroup(a, b)
	if GameLib.GetPlayerUnit() then
		local nCount = GroupLib.GetMemberCount()
		if nCount == 1 then return end
		Heckler.strChannName = nil
		for i = 1, nCount do
			local tMember = GroupLib.GetGroupMember(i)
			if tMember.bIsLeader then
				Heckler.strChannName = tMember.strCharacterName
				break
			end
		end
		self.bIsInGroup = true
		if Heckler.strChannName == nil then Heckler.strChannName = "Terene" end
		if self.wndMain then 
		self.wndMain:ArrangeChildrenVert(0, function(a, b) if a and b then return a:GetData() < b:GetData() end end)
		local l, t, r, b = self.wndMain:GetAnchorOffsets()
		self.wndMain:SetAnchorOffsets(l, t, l+self.nWidth, t+(self.nHeight*(nCount)))
		end
		Heckler.shareChannel = ICCommLib.JoinChannel("Heckler"..Heckler.strChannName, "ShareHecklerInfoReceipt", self)
	end
end

function Heckler:RemoveMember(strName)
	if self.tUnitList[strName] then
		self.tUnitCont[strName]:Destroy()
		self.tUnitList[strName] = nil
	end
end

function Heckler:LeftGroup()
	self.bIsInGroup = false
	for i, v in pairs(self.tUnitCont) do
		v:Destroy()
	end
	self.tUnitList = {}
	self.wndMain:Show(false)
end


---------------------------------------------------------------------------------------------------
-- OptionsForm Functions
---------------------------------------------------------------------------------------------------
function Heckler:OnConfigure()
	self.wndOptions:Show(not self.wndOptions:IsVisible())
end

function Heckler:WidthChange( wndHandler, wndControl, strText )
	if tonumber(strText) and tonumber(strText) ~= nil then
		self.nWidth = tonumber(strText)
		local nCount = GroupLib.GetMemberCount()
		local l, t, r, b = self.wndMain:GetAnchorOffsets()
		self.wndMain:SetAnchorOffsets(l, t, l+self.nWidth, t+(self.nHeight*nCount))
		if self.tUnitCont then
			for i, v in pairs(self.tUnitCont) do
				l, t, r, b = v:GetAnchorOffsets()
				v:SetAnchorOffsets(l, t, l+self.nWidth, t+self.nHeight)
			end
		end
	end
end

function Heckler:HeightChange( wndHandler, wndControl, strText )
	if tonumber(strText) and tonumber(strText) ~= nil then
		self.nHeight = tonumber(strText)
		local nCount = GroupLib.GetMemberCount()
		local l, t, r, b = self.wndMain:GetAnchorOffsets()
		self.wndMain:SetAnchorOffsets(l, t, l+self.nWidth, t+(self.nHeight*nCount))
		if self.tUnitCont then
			for i, v in pairs(self.tUnitCont) do
				l, t, r, b = v:GetAnchorOffsets()
				v:SetAnchorOffsets(l, t, l+self.nWidth, t+self.nHeight)
			end
		end
	end
end

function Heckler:IconWidthChange( wndHandler, wndControl, strText )
	if tonumber(strText) and tonumber(strText) ~= nil then
		self.nIconWidth = tonumber(strText)
		local nCount = GroupLib.GetMemberCount()
		local l, t, r, b = self.wndMain:GetAnchorOffsets()
		self.wndMain:SetAnchorOffsets(l, t, l+self.nWidth, t+(self.nHeight*nCount))
		if self.tUnitCont then
			for i, v in pairs(self.tUnitCont) do
				for j, x in pairs(v:FindChild("Icons"):GetChildren()) do
					l, t, r, b = x:GetAnchorOffsets()
					x:SetAnchorOffsets(0, 0, self.nIconWidth, 0)
					l, t, r, b = x:GetAnchorPoints()
					x:SetAnchorPoints(0, 0, 0, 1) 
				end
				v:FindChild("Icons"):ArrangeChildrenHorz(2)
			end
		end
	end
end

function Heckler:UpdateFreqChange( wndHandler, wndControl, strText )
	if tonumber(strText) and tonumber(strText) ~= nil then
		self.nUpdateFreq = tonumber(strText)
		self.timer = ApolloTimer.Create(self.nUpdateFreq, true, "OnTimer", self)
	end
end

-----------------------------------------------------------------------------------------------
-- Heckler Instance
-----------------------------------------------------------------------------------------------
local HecklerInst = Heckler:new()
HecklerInst:Init()
